@@ -15,8 +15,9 @@ use warnings;
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(debug $DEBUG carp_once);
 our %EXPORT_TAGS = (all => [ @EXPORT_OK ]);
-our $VERSION = '0.80';
+our $VERSION = '0.81';
 our $DUMPSIG;
+our $EM = 0;
 
 our ($DEBUG, $DEBUG_FH, %_CARPED, 
      $OLD_SIG__WARN__, $OLD_SIG__DIE__, $OLD_CARP_VERBOSE);
@@ -53,7 +54,11 @@ sub init {
 sub debug {
     my @msg = @_;
     # TODO: consider locking the filehandle or other synchronization
-    print {$DEBUG_FH} $$,' ',Forks::Super::Util::Ctime(),' ',@msg,"\n";
+    if ($EM) {
+        print {$DEBUG_FH} $$,'E ',Forks::Super::Util::Ctime(),' ',@msg,"\n";
+    } else {
+        print {$DEBUG_FH} $$,' ',Forks::Super::Util::Ctime(),' ',@msg,"\n";
+    }
     return;
 }
 
@@ -432,7 +437,7 @@ Forks::Super::Debug - debugging and logging routines for Forks::Super distro
 
 =head1 VERSION
 
-0.80
+0.81
 
 =head1 VARIABLES
 
@@ -541,7 +546,7 @@ Marty O'Brien, E<lt>mob@cpan.orgE<gt>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2009-2015, Marty O'Brien.
+Copyright (c) 2009-2016, Marty O'Brien.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
