@@ -28,7 +28,7 @@ eval 'use Devel::GlobalDestruction';
 our @ISA = qw(Exporter);
 our @EXPORT = qw(@ALL_JOBS %ALL_JOBS PREFORK POSTFORK 
                  POSTFORK_PARENT POSTFORK_CHILD);
-our $VERSION = '0.80';
+our $VERSION = '0.81';
 
 our (@ALL_JOBS, %ALL_JOBS, @ARCHIVED_JOBS, $WIN32_PROC, $WIN32_PROC_PID);
 our $OVERLOAD_ENABLED = 0;
@@ -286,7 +286,6 @@ sub _mark_complete {
     my $job = shift;
     $job->{end} = Time::HiRes::time();
     $job->{state} = 'COMPLETE';
-
     $job->run_callback('share');
     $job->run_callback('collect');
     $job->run_callback('finish');
@@ -297,6 +296,7 @@ sub _mark_reaped {
     my $job = shift;
     $job->{state} = 'REAPED';
     $job->{reaped} = Time::HiRes::time();
+    $job->run_callback('reaped');
     $? = defined $job->{status} ? $job->{status} : $job->{daemon} && 0;
     debug("Job $job->{pid} reaped") if $job->{debug};
     return;
@@ -551,6 +551,12 @@ sub launch {
 
     $_->() for @PREFORK;
 
+    if ($job->{emulate} ||
+        ($Forks::Super::EMULATION_MODE && !defined $job->{emulate})) {
+        debug("emulating child process in main process") if $job->{debug};
+        return $job->_emulate;
+    }
+
     my $pid = _robust_fork($job);
     if (!defined $pid) {
 	debug('launch(): CORE::fork() returned undefined!')
@@ -568,7 +574,8 @@ sub launch {
         $_->() for @POSTFORK_PARENT;
 
 	# parent
-	return _postlaunch_parent($pid, $job);
+	_postlaunch_parent1($pid, $job);
+        return $job->_postlaunch_parent2;
 
     } 
     if ($pid == 0) {
@@ -599,7 +606,7 @@ sub _robust_fork {
     my $job = shift;
     my $retries = $job->{retries} || 0;
 
-    # the other 17,000 lines in this distro are just a wrapper around this line:
+    # the other 21,000 lines in this distro are just a wrapper around this line:
 
     my $pid = _CORE_fork();
 
@@ -615,7 +622,14 @@ sub _robust_fork {
     return $pid;
 }
 
-sub _postlaunch_parent {
+sub _emulate {
+    my $job = shift;
+    require Forks::Super::Job::Emulate;
+    bless $job, 'Forks::Super::Job::Emulate';
+    return $job->emulate;
+}
+
+sub _postlaunch_parent1 {
     my ($pid, $job) = @_;
 
     $ALL_JOBS{$pid} = $job;
@@ -643,7 +657,8 @@ sub _postlaunch_parent {
     $job->{pid} ||= $pid;
     $job->{start} = Time::HiRes::time();
 
-    $job->_config_parent;
+    $job->_config_parent(1);
+    return;
     $job->run_callback('start');
 
     Forks::Super::handle_CHLD(-1);
@@ -660,6 +675,23 @@ sub _postlaunch_parent {
     return $OVERLOAD_ENABLED ? $job : $pid;
 }
 
+sub _postlaunch_parent2 {
+    my $job = shift;
+    $job->_config_parent(2);
+    $job->run_callback('start');
+    Forks::Super::handle_CHLD(-1);
+    if ($$ != $Forks::Super::MAIN_PID) {
+	# Forks::Super::fork call from a child.
+	$XSIG{CHLD}[-1] ||= \&Forks::Super::handle_CHLD;
+    }
+    if ($^O eq 'MSWin32') {
+	if ($job->{timeout} || $job->{expiration}) {
+	    $job->_read_signal_pid;
+	}
+    }
+    return $OVERLOAD_ENABLED ? $job : $job->{pid};
+}
+
 sub _postlaunch_daemon_parent {
     my ($job, $pid) = @_;
 
@@ -906,9 +938,12 @@ sub __adjust_cygwin_daemon_job {
 
 sub _postlaunch_child {
     my $job = shift;
-    if (defined &Forks::Super::init_child()) {
-	Forks::Super::init_child();
-    }
+
+    local *STDIN = *STDIN;
+    local *STDOUT = *STDOUT;
+    local *STDERR = *STDERR;
+
+    Forks::Super::init_child() unless $job->{is_emulation};
     $job->_config_child;
 
     local $ENV{_FORK_PPID} = $$;
@@ -982,7 +1017,7 @@ sub _postlaunch_child_to_exec {
     if ($Forks::Super::Debug::DEBUG) {
 	debug("waitpid $WIN32_PROC_PID ==> $z");
     }
-    deinit_child();
+    deinit_child() unless $job->{is_emulation};
     exit $c1 >> 8;
 }
 
@@ -995,7 +1030,7 @@ sub _postlaunch_child_to_proc {
     close $proch;
     my $c1 = $?;
     debug("Exit code of $$ was $c1 ", $c1>>8) if $job->{debug};
-    deinit_child();
+    deinit_child() unless $job->{is_emulation};
     exit $c1 >> 8;
 }
 
@@ -1024,10 +1059,11 @@ sub _postlaunch_child_to_cmd {
             croak "Forks::Super::Job::_postlaunch_child_to_cmd: ",
                 "Child process unable to create new fork to run cmd";
         }
-	if ($$ != $this_pid) {
+        if ($exec_pid == 0) {
 	    exec( @{$job->{cmd}} ) or
                 Carp::confess 'exec for cmd-style fork failed ';
 	}
+        $job->{debug} && debug("  exec pid is $exec_pid");
 	$job->set_signal_pid($exec_pid);
 	$job->{exec_pid} = $exec_pid;
 
@@ -1039,8 +1075,13 @@ sub _postlaunch_child_to_cmd {
 	debug("waitpid returned $z, exit code of $$ was $c1 ", $c1>>8)
 	    if $job->{debug};
     }
-    deinit_child();
-    exit $c1 >> 8;
+    if ($job->{is_emulation}) {
+        Forks::Super::Job::Ipc::_close_child_fh($job);
+        Forks::Super::Sigchld::_preliminary_reap($job,$c1);
+    } else {
+        deinit_child();
+        exit $c1 >> 8;
+    }
 }
 
 sub _postlaunch_child_to_remote_cmd {
@@ -1156,9 +1197,17 @@ sub _postlaunch_child_to_sub {
 	}
 	debug("Job $$ subroutine call has completed");
     }
+    if ($job->{is_emulation}) {
+        my $status = 0;
+        if ($error) {
+            $job->{error} = $error;
+            $status = 255 << 8;
+        }
+        return Forks::Super::Sigchld::_preliminary_reap($job,$status);
+    }
     deinit_child();
     if ($error) {
-	die $error,"\n";
+        die $error,"\n";
     }
     exit 0;
 }
@@ -1176,7 +1225,6 @@ sub _launch_from_child {
     if ($Forks::Super::CHILD_FORK_OK == 0) {
 	carp "Forks::Super::fork() not allowed in child process $$ ",
 	        "while \$Forks::Super::CHILD_FORK_OK is not set!";
-
 	return;
     } elsif ($Forks::Super::CHILD_FORK_OK < 0) {
 	carp "Forks::Super::fork() call not allowed in child process $$ ",
@@ -1638,12 +1686,12 @@ sub _preconfig_start_time {
 	$start_after
 	    = Time::HiRes::time() 
 	    + Forks::Super::Job::Timeout::_time_from_natural_language(
-	    $job->{delay}, 1);
+	            $job->{delay}, 1);
     }
     if (defined $job->{start_after}) {
-	my $start_after2 
-	    = Forks::Super::Job::Timeout::_time_from_natural_language(
-	    $job->{start_after}, 0);
+	my $start_after2 =
+            Forks::Super::Job::Timeout::_time_from_natural_language(
+                $job->{start_after}, 0);
 	if ($start_after < $start_after2) {
 	    $start_after = $start_after2 
 	}
@@ -1651,7 +1699,8 @@ sub _preconfig_start_time {
     if ($start_after) {
 	$job->{start_after} = $start_after;
 	delete $job->{delay};
-	debug('_can_launch(): start delay requested.')
+	debug('_can_launch(): start delay until ' 
+              . localtime($job->{start_after}) . ' requested.')
 	    if $job->{debug};
     }
     return;
@@ -1740,10 +1789,11 @@ sub _resolve_names {
 # child is successfully launched.
 #
 sub _config_parent {
-    my $job = shift;
-    $job->_config_fh_parent;
-    $job->_config_timeout_parent;
-    if ($Forks::Super::SysInfo::CONFIG{'getpgrp'}) {
+    my ($job,$step) = @_;
+    $step = $step || 1;
+    $job->_config_fh_parent($step);
+    $job->_config_timeout_parent($step);
+    if ($step < 2 && $Forks::Super::SysInfo::CONFIG{'getpgrp'}) {
 	# when  timeout =>   or   expiration =>  is used,
 	# PGID of child will be set to child PID
 	if (defined($job->{timeout}) || defined($job->{expiration})) {
@@ -2276,7 +2326,7 @@ Forks::Super::Job - object representing a background task
 
 =head1 VERSION
 
-0.80
+0.81
 
 =head1 SYNOPSIS
 
@@ -3086,7 +3136,7 @@ Marty O'Brien, E<lt>mob@cpan.orgE<gt>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2009-2015, Marty O'Brien.
+Copyright (c) 2009-2016, Marty O'Brien.
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself, either Perl version 5.8.8 or,
