@@ -33,7 +33,7 @@ $| = 1;
 
 our @ISA = qw(Exporter);
 our @EXPORT = qw(close_fh);
-our $VERSION = '0.80';
+our $VERSION = '0.81';
 our $NO_README = 0;
 
 our (%FILENO, %SIG_OLD, $IPC_COUNT, $IPC_DIR_DEDICATED,
@@ -278,9 +278,11 @@ sub _safeopen_open {
     my ($fh, $open2, $open3) = @_;
     my $result;
     if (defined $open3) {
+        no warnings 'io';
 	$result = CORE::open($fh, $open2, $open3);      ## no critic (BriefOpen)
 	$_[0] = $fh;
     } else {
+        no warnings 'io';
 	$result = CORE::open($fh, $open2);              ## no critic (BriefOpen)
 	$_[0] = $fh;
     }
@@ -701,6 +703,9 @@ sub _preconfig_fh_files {
 	    if (_safeopen($job, my $fh, '>', $config->{f_in})) {
 		print $fh $config->{stdin};
 		_close($fh);
+                debug("Stored ",length($config->{stdin})," bytes in ",
+                      $config->{f_in}," as child STDIN")
+                    if $job->{debug};
 	    } else {
 		carp 'Forks::Super::Job::_preconfig_fh: ',
 		    "scalar standard input not available in child: $!\n";
@@ -1318,7 +1323,7 @@ sub _END_background_cleanup1 {
 		$deleted{$ipc_file} = delete $IPC_FILES{$ipc_file};
 	    } else {
 		if ($DEBUG) {
-		    print STDERR "Delete $ipc_file not ok: $!\n";
+		    print STDERR "Delete $ipc_file failed: $!\n";
 		}
 		warn 'Forks::Super::END_cleanup: ',
 		    "error disposing of ipc file $ipc_file: $z/$!\n";
@@ -1403,6 +1408,7 @@ sub END_cleanup_MSWin32 {
     # XXX - is this ok? what if perl script is communicating with a socket?
     use POSIX ();
     for (0,1,3..999) {
+        no warnings;
 	POSIX::close($_);
     }
 
@@ -1834,12 +1840,14 @@ sub __config_fh_parent_stderr_file {
 sub Forks::Super::Job::_config_fh_parent {
     my $job = shift;
     return if not defined $job->{fh_config};
+    my $step = shift || 0;
 
     # _trap_signals(); # XXX - is this necessary?
     my $fh_config = $job->{fh_config};
 
     # set up stdin first.
-    _config_fh_parent_stdin($job);
+    _config_fh_parent_stdin($job) if $step != 2;
+    return if $step == 1;
     _config_fh_parent_stdout($job);
     _config_fh_parent_stderr($job);
 
@@ -1921,20 +1929,24 @@ sub __config_fh_child_stdin_scalar {
 	    carp 'Forks::Super::Job::Ipc::_config_fh_child_stdin: ',
 		    "Error initializing scalar STDIN in child $$: $!\n";
 	}
-
-    } elsif (!(_safeopen($job, $fh, '<', $fh_config->{f_in}, no_layers => 1))) {
-	carp 'Forks::Super::Job::Ipc::_config_fh_child_stdin(): ',
-		"Error initializing scalar STDIN in child $$: $!\n";
-    } elsif (!(_safeopen($job, *STDIN, '<&', $fh))) {
-	push @{$job->{child_fh_close}}, $fh;
-	carp 'Forks::Super::Job::Ipc::_config_fh_child_stdin(): ',
-		"Error initializing scalar STDIN in child $$: $!\n";
-    } else {
-	push @{$job->{child_fh_close}}, $fh;
+        return;
     }
+    if (!_safeopen($job, $fh, '<', $fh_config->{f_in}, no_layers => 1)) {
+	carp 'Forks::Super::Job::Ipc::_config_fh_child_stdin(): ',
+            "Error initializing scalar STDIN in child $$: $!\n";
+        return;
+    }
+    if (!_safeopen($job, *STDIN, '<&', $fh)) {
+	carp 'Forks::Super::Job::Ipc::_config_fh_child_stdin(): ',
+        "Error initializing scalar STDIN in child $$: $!\n";
+        close $fh;
+        return;
+    }
+    push @{$job->{child_fh_close}}, $fh;
     return;
 }
 
+
 sub __config_fh_child_stdin_sockets {
     my $job = shift;
     my $fh_config = $job->{fh_config};
@@ -2265,9 +2277,11 @@ sub Forks::Super::Job::_config_fh_child {
 
     # "a tie in the parent should not be allowed to cause problems"
     # according to IPC::Open3
-    untie *STDIN;
-    untie *STDOUT;
-    untie *STDERR;
+    if (!$job->{is_emulation}) {
+        untie *STDIN;
+        untie *STDOUT;
+        untie *STDERR;
+    }
 
     # track handles to close when the child exits
     $job->{child_fh_close} = [];
@@ -2275,7 +2289,9 @@ sub Forks::Super::Job::_config_fh_child {
     if ($job->{style} eq 'cmd' || $job->{style} eq 'exec') {
 	if (&IS_WIN32 && Forks::Super::Config::CONFIG('filehandles')) {
 	    return _config_cmd_fh_child($job);
-	}
+	} elsif ($job->{is_emulation}) {
+            return _config_cmd_fh_child($job);
+        }
     }
 
     _config_fh_child_stdout($job);
@@ -2305,6 +2321,8 @@ sub Forks::Super::Job::_config_fh_child {
     return;
 }
 
+# not-idiot-proof compression of a list of command arguments
+# into a single string with escaped metacharacters
 sub _collapse_command {
     my @cmd = @_;
     if (@cmd <= 1) {
@@ -2324,12 +2342,12 @@ sub _collapse_command {
 	    push @new_cmd, "\"$cmd2\"";
 	}
     }
-    @cmd = ();
-    push @cmd, (join ' ', @new_cmd);
+    @cmd = (join ' ', @new_cmd);
     return @cmd;
 }
 
 # MSWin32 has trouble using the open '>&' and open '<&' syntax.
+# We should also use shell I/O redirection in emulation mode
 sub _config_cmd_fh_child {
     my $job = shift;
     my $fh_config = $job->{fh_config};
@@ -2349,9 +2367,8 @@ sub _config_cmd_fh_child {
     if ($fh_config->{err} && $fh_config->{f_err} && !$fh_config->{join}) {
 	$cmd[0] .= " 2>\"$fh_config->{f_err}\"";
     }
-    if (!$fh_config->{in}) {
-	close STDIN;
-    } elsif ($fh_config->{f_in}) {
+
+    if ($fh_config->{f_in}) {
 
 	# standard input must be specified before the first pipe char,
 	# if any (How do you distinguish pipes that are
@@ -2384,6 +2401,8 @@ sub _config_cmd_fh_child {
 	    	"child was not able to detect STDIN file $fh_config->{f_in}. ",
 	    	"Child may not have any input to read.\n";
 	}
+    } elsif (!$job->{is_emulation}) {
+        close STDIN;
     }
     debug("config_cmd_fh_config(): child cmd is   $cmd[0]  ")
 	if $job->{debug};
@@ -2454,10 +2473,8 @@ sub _close {
     if (!defined $$handle->{opened}) {
 	# this method should only be used with I/O handles
 	# opened by Forks::Super
-	if (1 || $DEBUG) {
-	    Carp::cluck 'Forks::Super::Job::Ipc::_close ',
+        Carp::cluck 'Forks::Super::Job::Ipc::_close ',
 	    "called on unrecognized file handle $handle\n";
-	}
     }
 
     if (is_socket($handle)) {
@@ -2468,7 +2485,7 @@ sub _close {
     my $z = 0;
     my $is_tied = !!tied *$handle;
 
-    delete $__OPEN_FH{fileno($handle)};
+    fileno($handle) && delete $__OPEN_FH{fileno($handle)};
     if ($is_tied) {
 	#my $th = tied *$handle;
 	#my $z = $th->CLOSE;
@@ -3237,6 +3254,10 @@ sub init_child {
 sub deinit_child {
     use Data::Dumper;
     my $job = Forks::Super::Job->this;
+    if ($job->{is_emulation}) {
+        local *STDERR = *$Forks::Super::Debug::DEBUG_FH;
+        Carp::cluck("ZZZZ deinit_child called for emulated job!");
+    }
     if (defined($job->{share}) && defined($job->{share_ipc})) {
 	if (open my $fh, '>', $job->{share_ipc}) {
 	    print $fh Data::Dumper::Dumper( $job->{share} );
@@ -3256,7 +3277,12 @@ sub deinit_child {
 	unlink @IPC_FILES;
 	@IPC_FILES = ();
     }
+    _close_child_fh($job);
+    return;
+}
 
+sub _close_child_fh {
+    my $job = shift;
     my %closed = ();
     foreach my $fh (@{$job->{child_fh_close}}, @SAFEOPENED) {
 	if (ref $fh eq 'ARRAY') {
@@ -3269,7 +3295,6 @@ sub deinit_child {
 	    }
 	    next;
 	}
-
 	next if $closed{$fh}++ || $$fh->{closed};
 	close $fh;
     }
@@ -3299,3 +3324,37 @@ sub _ipcHandleToString {
 }
 
 1;
+
+=head1 NAME
+
+Forks::Super::Job::Ipc - interprocess communication routines for Forks::Super
+
+=head1 VERSION
+
+0.81
+
+=head1 DESCRIPTION
+
+C<Forks::Super::Job::Ipc> is part of the L<Forks::Super|Forks::Super>
+distribution. The functions and variables in this package manage
+communication between parent and child processes.
+
+This package is heavily used by the L<Forks::Super::Job|Forks::Super::Job>
+package, but there is little reason for a L<Forks::Super|Forks::Super>
+user to call this package's functions directly.
+
+=head1 AUTHOR
+
+Marty O'Brien, E<lt>mob@cpan.orgE<gt>
+
+=head1 LICENSE AND COPYRIGHT
+
+Copyright (c) 2009-2016, Marty O'Brien.
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either Perl version 5.8.8 or,
+at your option, any later version of Perl 5 you may have available.
+
+See http://dev.perl.org/licenses/ for more information.
+
+=cut
