@@ -27,7 +27,7 @@ if (!&Forks::Super::Util::IS_WIN32ish) {
 #   http://msdn.microsoft.com/en-us/library/ms684847(VS.85).aspx
 
 
-our $VERSION = '0.80';
+our $VERSION = '0.81';
 our ($_THREAD_API, $_THREAD_API_INITIALIZED, %SYSTEM_INFO);
 
 ##################################################################
@@ -429,7 +429,8 @@ sub terminate_process {
     if ($procHandle) {
 	my $z = win32api('TerminateProcess',$procHandle,$exitCode || 0);
 	if (!$z) {
-	    carp "Forks::Super::Win32: terminate_process: $^E\n";
+	    carp "Forks::Super::Win32: terminate_process: \$^E=",
+                    0+$^E," $^E\n";
 	} else {
 	    return $z;
 	}
