@@ -10,7 +10,7 @@ use Carp;
 use strict;
 use warnings;
 
-our $VERSION = '0.80';
+our $VERSION = '0.81';
 
 # "protocols" for serializing data and the methods used
 # to carry out the serialization
@@ -28,17 +28,15 @@ my %serialization_dispatch = (
 	encode => sub { return Data::Dumper::Dumper( \@_ ) },
 	decode => sub {
 	    my ($data,$job,$VAR1) = @_;
-	    if (${^TAINT}) {
-		if ($job->{untaint}) {
-		    ($data) = $data =~ /(.*)/s;
-		} else {
-		    carp 'Forks::Super::bg_eval/bg_qx(): ',
+            if ($job->{untaint}) {
+                ($data) = $data =~ /(.*)/s;
+            } elsif (${^TAINT}) {
+                carp 'Forks::Super::bg_eval/bg_qx(): ',
 		        'Using Data::Dumper for serialization, which cannot ',
 		        "operate on 'tainted' data. Use bg_eval {...} ",
 		        '{untaint => 1} or bg_qx COMMAND, ',
 		        "{untaint => 1} to retrieve the result.\n";
-		    return;
-		}
+                return;
 	    }
 	    my $decoded = eval "$data";    ## no critic (StringyEval)
 	    return $decoded;
@@ -186,7 +184,7 @@ Forks::Super::LazyEval::BackgroundArray
 
 =head1 VERSION
 
-0.80
+0.81
 
 =head1 DESCRIPTION
 
@@ -202,7 +200,7 @@ Marty O'Brien, E<lt>mob@cpan.orgE<gt>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2010-2013, Marty O'Brien.
+Copyright (c) 2010-2016, Marty O'Brien.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
