@@ -59,7 +59,7 @@ use overload # XXX - what is overloading for? is it necessary?
 		         : atan2($_[0]->_fetch, $_[1]) }
 ;
 
-our $VERSION = '0.80';
+our $VERSION = '0.81';
 
 # "protocols" for serializing data and the methods used
 # to carry out the serialization
@@ -77,18 +77,16 @@ my %serialization_dispatch = (
 	encode => sub { return Data::Dumper::Dumper($_[0]) },
 	decode => sub {
 	    my ($data,$job,$VAR1) = @_;
-	    if (${^TAINT}) {
-		if ($job->{untaint}) {
-		    ($data) = $data =~ /(.*)/s;
-		} else {
-		    carp 'Forks::Super::bg_eval/bg_qx(): ',
-		        'Using Data::Dumper for serialization, which cannot ',
-		        "operate on 'tainted' data. Use bg_eval {...} ",
-		        '{untaint => 1} or bg_qx COMMAND, ',
-		        "{untaint => 1} to retrieve the result.\n";
-		    return;
-		}
-	    }
+            if ($job->{untaint}) {
+                ($data) = $data =~ /(.*)/s;
+            } elsif (${^TAINT}) {
+                carp 'Forks::Super::bg_eval/bg_qx(): ',
+                        'Using Data::Dumper for serialization, which cannot ',
+                        "operate on 'tainted' data. Use bg_eval {...} ",
+                        '{untaint => 1} or bg_qx COMMAND, ',
+                        "{untaint => 1} to retrieve the result.\n";
+                return;
+            }
 	    my $decoded = eval "$data";    ## no critic (StringyEval)
 	    return $decoded;
 	}
@@ -266,7 +264,7 @@ Forks::Super::LazyEval::BackgroundScalar
 
 =head1 VERSION
 
-0.80
+0.81
 
 =head1 DESCRIPTION
 
@@ -282,7 +280,7 @@ Marty O'Brien, E<lt>mob@cpan.orgE<gt>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2010-2015, Marty O'Brien.
+Copyright (c) 2010-2016, Marty O'Brien.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
