@@ -56,19 +56,25 @@ eval { use Time::HiRes; 1 }
 $| = 1;
 
 $^T = Time::HiRes::time();
+BEGIN {
 if (${^TAINT}) {
     if ($^O eq 'MSWin32') {
 	($ENV{PATH}) = $ENV{PATH} =~ /(.*)/;
     } else {
 	$ENV{PATH} = '/bin:/usr/bin:/usr/local/bin';
     }
-    ($^X)=$^X=~/(.*)/;
+    delete $ENV{ENV};
+    use Config;
+    $^X = $Config::Config{perlpath};
     ($ENV{HOME})=$ENV{HOME}=~/(.*)/;
     $ENV{PWD} = Cwd::getcwd();
     ($ENV{PWD})=$ENV{PWD}=~/(.*)/;
-    ($ENV{TEMP})=$ENV{TEMP}=~/(.*)/;
+    $ENV{TEMP} = "C:/Temp";
+    ($ENV{IPC_DIR}) = $ENV{IPC_DIR} =~ /(.*)/
+	if defined $ENV{IPC_DIR};
     @ARGV = map { /(.*)/ } @ARGV;
 }
+}
 
 my @use_libs = qw(blib/lib blib/arch);
 my @perl_opts = ();
@@ -261,7 +267,7 @@ sub main {
 		return if process_test_output($reap) eq 'ABORT';
 		$reap = -1;
 		shift @j;
-		$waitproc = $inorder ? $j[0] : -1;
+		$waitproc = $inorder && @j ? $j[0] : -1;
 		$reap = waitpid $waitproc, WNOHANG;
 	    }
 	}
@@ -279,7 +285,7 @@ sub main {
 	    while (Forks::Super::Util::isValidPid($pid)) {
 		shift @j;
 		return if process_test_output($pid) eq 'ABORT';
-		$pid = waitpid $j[0], 0;
+		$pid = @j ? waitpid $j[0], 0 : -1;
 	    }
 	} else {
 	    my $pid = wait;
@@ -316,6 +322,7 @@ sub launch_test_file {
 	$test_harness = "test_harness($test_verbose";
 	$test_harness .= ",'$_'" foreach @use_libs;
 	$test_harness .= ')';
+	my @extra_opts = _get_perl_opts($test_file);
 	if ($] < 5.007) {
 	    @cmd = ($^X, '-Iblib/lib', '-Iblib/arch',
 		    '-e', 'use Test::Harness qw(&runtests $verbose);',
@@ -323,8 +330,8 @@ sub launch_test_file {
 		    '-e', 'runtests @ARGV',
 		    $test_file);
 	} else {
-	    @cmd = ($^X, '-MExtUtils::Command::MM', '-e',
-		    $test_harness, $test_file);
+	    @cmd = ($^X, '-MExtUtils::Command::MM',
+                    '-e', $test_harness, $test_file);
 	}
     } else {
 	my @extra_opts = _get_perl_opts($test_file);
@@ -664,9 +671,7 @@ sub _summarize_failures {
 	    $a+0<=>$b+0 || $a cmp $b
 			     } keys %{$fail{$test_file}}) {
 	    print "\t$test_file#$test_no ";
-	    if ($fail{$test_file}{$test_no} == 1) {
-		print "1 time\n";
-	    } else {
+	    if ($fail{$test_file}{$test_no} > 1) {
 		print "$fail{$test_file}{$test_no} times\n";
 	    }
 	}
@@ -819,7 +824,7 @@ forked_harness.pl - run tests in parallel with Forks::Super
 
 =head1 VERSION
 
-0.80
+0.81
 
 =head1 SYNOPSIS
 
